# -*- coding: utf-8 -*-
# This file was generated

from enum import Enum


class AcquisitionStatus(Enum):
    COMPLETE = 1
    IN_PROGRESS = 0
    STATUS_UNKNOWN = -1


class AcquisitionType(Enum):
    NORMAL = 0
    r'''
    Sets the digitizer to normal resolution mode. The digitizer can use real-time sampling or equivalent-time sampling.
    '''
    FLEXRES = 1001
    r'''
    Sets the digitizer to flexible resolution mode if supported.  The digitizer uses different hardware configurations to change the resolution depending on the sampling rate used.
    '''
    DDC = 1002
    r'''
    Sets the digitizer to DDC mode on the NI 5620/5621.
    '''


class ArrayMeasurement(Enum):
    NO_MEASUREMENT = 4000
    r'''
    None
    '''
    LAST_ACQ_HISTOGRAM = 4001
    r'''
    Last Acquisition Histogram
    '''
    FFT_PHASE_SPECTRUM = 4002
    r'''
    FFT Phase Spectrum
    '''
    FFT_AMP_SPECTRUM_VOLTS_RMS = 4003
    r'''
    FFT Amp. Spectrum (Volts RMS)
    '''
    MULTI_ACQ_VOLTAGE_HISTOGRAM = 4004
    r'''
    Multi Acquisition Voltage Histogram
    '''
    MULTI_ACQ_TIME_HISTOGRAM = 4005
    r'''
    Multi Acquisition Time Histogram
    '''
    ARRAY_INTEGRAL = 4006
    r'''
    Array Integral
    '''
    DERIVATIVE = 4007
    r'''
    Derivative
    '''
    INVERSE = 4008
    r'''
    Inverse
    '''
    HANNING_WINDOW = 4009
    r'''
    Hanning Window
    '''
    FLAT_TOP_WINDOW = 4010
    r'''
    Flat Top Window
    '''
    POLYNOMIAL_INTERPOLATION = 4011
    r'''
    Polynomial Interpolation
    '''
    MULTIPLY_CHANNELS = 4012
    r'''
    Multiply Channels
    '''
    ADD_CHANNELS = 4013
    r'''
    Add Channels
    '''
    SUBTRACT_CHANNELS = 4014
    r'''
    Subtract Channels
    '''
    DIVIDE_CHANNELS = 4015
    r'''
    Divide Channels
    '''
    MULTI_ACQ_AVERAGE = 4016
    r'''
    Multi Acquisition Average
    '''
    BUTTERWORTH_FILTER = 4017
    r'''
    Butterworth IIR Filter
    '''
    CHEBYSHEV_FILTER = 4018
    r'''
    Chebyshev IIR Filter
    '''
    FFT_AMP_SPECTRUM_DB = 4019
    r'''
    FFT Amp. Spectrum (dB)
    '''
    HAMMING_WINDOW = 4020
    r'''
    Hamming Window
    '''
    WINDOWED_FIR_FILTER = 4021
    r'''
    FIR Windowed Filter
    '''
    BESSEL_FILTER = 4022
    r'''
    Bessel IIR Filter
    '''
    TRIANGLE_WINDOW = 4023
    r'''
    Triangle Window
    '''
    BLACKMAN_WINDOW = 4024
    r'''
    Blackman Window
    '''
    ARRAY_OFFSET = 4025
    r'''
    Array Offset
    '''
    ARRAY_GAIN = 4026
    r'''
    Array Gain
    '''


class CableSenseMode(Enum):
    DISABLED = 0
    r'''
    The oscilloscope is not configured to emit a CableSense signal.
    '''
    ON_DEMAND = 1
    r'''
    The oscilloscope is configured to emit a single CableSense pulse.
    '''


class _CalibrationTypes(Enum):
    SELF = 1
    EXTERNAL = 0
    MANUFACTURE = 2


class ClearableMeasurement(Enum):
    ALL_MEASUREMENTS = 10000
    MULTI_ACQ_VOLTAGE_HISTOGRAM = 4004
    MULTI_ACQ_TIME_HISTOGRAM = 4005
    MULTI_ACQ_AVERAGE = 4016
    FREQUENCY = 2
    AVERAGE_FREQUENCY = 1016
    FFT_FREQUENCY = 1008
    PERIOD = 3
    AVERAGE_PERIOD = 1015
    RISE_TIME = 0
    FALL_TIME = 1
    RISE_SLEW_RATE = 1010
    FALL_SLEW_RATE = 1011
    OVERSHOOT = 18
    PRESHOOT = 19
    VOLTAGE_RMS = 4
    VOLTAGE_CYCLE_RMS = 16
    AC_ESTIMATE = 1012
    FFT_AMPLITUDE = 1009
    VOLTAGE_AVERAGE = 10
    VOLTAGE_CYCLE_AVERAGE = 17
    DC_ESTIMATE = 1013
    VOLTAGE_MAX = 6
    VOLTAGE_MIN = 7
    VOLTAGE_PEAK_TO_PEAK = 5
    VOLTAGE_HIGH = 8
    VOLTAGE_LOW = 9
    AMPLITUDE = 15
    VOLTAGE_TOP = 1007
    VOLTAGE_BASE = 1006
    VOLTAGE_BASE_TO_TOP = 1017
    WIDTH_NEG = 11
    WIDTH_POS = 12
    DUTY_CYCLE_NEG = 13
    DUTY_CYCLE_POS = 14
    INTEGRAL = 1005
    AREA = 1003
    CYCLE_AREA = 1004
    TIME_DELAY = 1014
    PHASE_DELAY = 1018
    LOW_REF_VOLTS = 1000
    MID_REF_VOLTS = 1001
    HIGH_REF_VOLTS = 1002
    VOLTAGE_HISTOGRAM_MEAN = 2000
    VOLTAGE_HISTOGRAM_STDEV = 2001
    VOLTAGE_HISTOGRAM_MEDIAN = 2003
    VOLTAGE_HISTOGRAM_MODE = 2010
    VOLTAGE_HISTOGRAM_MAX = 2005
    VOLTAGE_HISTOGRAM_MIN = 2006
    VOLTAGE_HISTOGRAM_PEAK_TO_PEAK = 2002
    VOLTAGE_HISTOGRAM_MEAN_PLUS_STDEV = 2007
    VOLTAGE_HISTOGRAM_MEAN_PLUS_2_STDEV = 2008
    VOLTAGE_HISTOGRAM_MEAN_PLUS_3_STDEV = 2009
    VOLTAGE_HISTOGRAM_HITS = 2004
    VOLTAGE_HISTOGRAM_NEW_HITS = 2011
    TIME_HISTOGRAM_MEAN = 3000
    TIME_HISTOGRAM_STDEV = 3001
    TIME_HISTOGRAM_MEDIAN = 3003
    TIME_HISTOGRAM_MODE = 3010
    TIME_HISTOGRAM_MAX = 3005
    TIME_HISTOGRAM_MIN = 3006
    TIME_HISTOGRAM_PEAK_TO_PEAK = 3002
    TIME_HISTOGRAM_MEAN_PLUS_STDEV = 3007
    TIME_HISTOGRAM_MEAN_PLUS_2_STDEV = 3008
    TIME_HISTOGRAM_MEAN_PLUS_3_STDEV = 3009
    TIME_HISTOGRAM_HITS = 3004
    TIME_HISTOGRAM_NEW_HITS = 3011


class FIRFilterWindow(Enum):
    NONE = 0
    r'''
    No window.
    '''
    HANNING = 409
    r'''
    Specifies a Hanning window.
    '''
    FLAT_TOP = 410
    r'''
    Specifies a Flat Top window.
    '''
    HAMMING = 420
    r'''
    Specifies a Hamming window.
    '''
    TRIANGLE = 423
    r'''
    Specifies a Triangle window.
    '''
    BLACKMAN = 424
    r'''
    Specifies a Blackman window.
    '''


class FetchRelativeTo(Enum):
    READ_POINTER = 388
    r'''
    The read pointer is set to zero when a new acquisition is initiated. After every fetch the read pointer is incremeted to be the sample after the last sample retrieved.  Therefore, you can repeatedly fetch relative to the read pointer for a continuous acquisition program.
    '''
    PRETRIGGER = 477
    r'''
    Fetches relative to the first pretrigger point requested with configure_horizontal_timing.
    '''
    NOW = 481
    r'''
    Fetch data at the last sample acquired.
    '''
    START = 482
    r'''
    Fetch data starting at the first point sampled by the digitizer.
    '''
    TRIGGER = 483
    r'''
    Fetch at the first posttrigger sample.
    '''


class FilterType(Enum):
    LOWPASS = 0
    r'''
    Specifies lowpass as the filter type.
    '''
    HIGHPASS = 1
    r'''
    Specifies highpass as the filter type.
    '''
    BANDPASS = 2
    r'''
    Specifies bandpass as the filter type.
    '''
    BANDSTOP = 3
    r'''
    Specifies bandstop as the filter type.
    '''


class FlexFIRAntialiasFilterType(Enum):
    FOURTYEIGHT_TAP_STANDARD = 0
    r'''
    This filter is optimized for alias protection and frequency-domain flatness
    '''
    FOURTYEIGHT_TAP_HANNING = 1
    r'''
    This filter is optimized for the lowest possible bandwidth for a 48 tap filter and maximizes the SNR
    '''
    SIXTEEN_TAP_HANNING = 2
    r'''
    This filter is optimized for the lowest possible bandwidth for a 16 tap filter and maximizes the SNR
    '''
    EIGHT_TAP_HANNING = 3
    r'''
    This filter is optimized for the lowest possible bandwidth for a 8 tap filter and maximizes the SNR
    '''


class GlitchCondition(Enum):
    GREATER = 2
    r'''
    Trigger on pulses with a duration greater than the specified glitch width.
    '''
    LESS = 1
    r'''
    Trigger on pulses with a duration shorter than the specified glitch width.
    '''


class GlitchPolarity(Enum):
    POSITIVE = 1
    r'''
    Trigger on pulses of positive polarity relative to the trigger threshold.
    '''
    NEGATIVE = 2
    r'''
    Trigger on pulses of negative polarity relative to the trigger threshold.
    '''
    EITHER = 3
    r'''
    Trigger on pulses of either positive or negative polarity.
    '''


class Option(Enum):
    SELF_CALIBRATE_ALL_CHANNELS = 0
    r'''
    Self Calibrating all Channels
    '''
    RESTORE_EXTERNAL_CALIBRATION = 1
    r'''
    Restore External Calibration.
    '''


class PercentageMethod(Enum):
    LOWHIGH = 0
    r'''
    Specifies that the reference level percentages should be computed using
    the low/high method,
    '''
    MINMAX = 1
    r'''
    Reference level percentages are computed using the min/max method.
    '''
    BASETOP = 2
    r'''
    Reference level percentages are computed using the base/top method.
    '''


class RISMethod(Enum):
    EXACT_NUM_AVERAGES = 1
    r'''
    Acquires exactly the specified number of records for each bin in the RIS acquisition.  An error is returned from the fetch method if the RIS acquisition does not successfully acquire the specified number of waveforms within the timeout period.  You may call the fetch method again to allow more time for the acquisition to finish.
    '''
    MIN_NUM_AVERAGES = 2
    r'''
    Each RIS sample is the average of a least a minimum number of randomly
    distributed points.
    '''
    INCOMPLETE = 3
    r'''
    Returns the RIS waveform after the specified timeout even if it is incomplete.  If no waveforms have been acquired in certain bins, these bins will have a NaN (when fetching scaled data) or a zero (when fetching binary data). A warning (positive error code) is returned from the fetch method if the RIS acquisition did not finish.  The acquisition aborts when data is returned.
    '''
    LIMITED_BIN_WIDTH = 5
    r'''
    Limits the waveforms in the various bins to be within 200 ps of the center of the bin.
    '''


class RefLevelUnits(Enum):
    VOLTS = 0
    r'''
    Specifies that the reference levels are given in units of volts.
    '''
    PERCENTAGE = 1
    r'''
    (Default) Specifies that the reference levels are given in percentage
    units.
    '''


class RefTriggerDetectorLocation(Enum):
    ANALOG_DETECTION_CIRCUIT = 0
    r'''
    use the hardware analog circuitry to implement the reference trigger.  This option will trigger before any onboard signal processing.
    '''
    DDC_OUTPUT = 1
    r'''
    use the onboard signal processing logic to implement the reference trigger.  This option will trigger based on the onboard signal processed data.
    '''


class RuntPolarity(Enum):
    POSITIVE = 1
    r'''
    Trigger on pulses of positive polarity relative to runt_low_threshold that do not cross runt_high_threshold.
    '''
    NEGATIVE = 2
    r'''
    Trigger on pulses of negative polarity relative to runt_high_threshold that do not cross runt_low_threshold.
    '''
    EITHER = 3
    r'''
    Trigger on pulses of either positive or negative polarity.
    '''


class RuntTimeCondition(Enum):
    NONE = 0
    r'''
    Time qualification is disabled. Trigger on runt pulses based solely on the voltage level of the pulses.
    '''
    WITHIN = 1
    r'''
    Trigger on pulses that, in addition to meeting runt voltage criteria, have a duration within the range bounded by runt_time_low_limit and runt_time_high_limit.
    '''
    OUTSIDE = 2
    r'''
    Trigger on pulses that, in addition to meeting runt voltage criteria, have a duration not within the range bounded by runt_time_low_limit and runt_time_high_limit.
    '''


class ScalarMeasurement(Enum):
    NO_MEASUREMENT = 4000
    r'''
    None
    '''
    RISE_TIME = 0
    FALL_TIME = 1
    FREQUENCY = 2
    PERIOD = 3
    VOLTAGE_RMS = 4
    VOLTAGE_PEAK_TO_PEAK = 5
    VOLTAGE_MAX = 6
    VOLTAGE_MIN = 7
    VOLTAGE_HIGH = 8
    VOLTAGE_LOW = 9
    VOLTAGE_AVERAGE = 10
    WIDTH_NEG = 11
    WIDTH_POS = 12
    DUTY_CYCLE_NEG = 13
    DUTY_CYCLE_POS = 14
    AMPLITUDE = 15
    VOLTAGE_CYCLE_RMS = 16
    VOLTAGE_CYCLE_AVERAGE = 17
    OVERSHOOT = 18
    PRESHOOT = 19
    LOW_REF_VOLTS = 1000
    MID_REF_VOLTS = 1001
    HIGH_REF_VOLTS = 1002
    AREA = 1003
    CYCLE_AREA = 1004
    INTEGRAL = 1005
    VOLTAGE_BASE = 1006
    VOLTAGE_TOP = 1007
    FFT_FREQUENCY = 1008
    FFT_AMPLITUDE = 1009
    RISE_SLEW_RATE = 1010
    FALL_SLEW_RATE = 1011
    AC_ESTIMATE = 1012
    DC_ESTIMATE = 1013
    TIME_DELAY = 1014
    AVERAGE_PERIOD = 1015
    AVERAGE_FREQUENCY = 1016
    VOLTAGE_BASE_TO_TOP = 1017
    PHASE_DELAY = 1018


class TerminalConfiguration(Enum):
    SINGLE_ENDED = 0
    r'''
    Channel is single ended
    '''
    UNBALANCED_DIFFERENTIAL = 1
    r'''
    Channel is unbalanced differential
    '''
    DIFFERENTIAL = 2
    r'''
    Channel is differential
    '''


class TriggerCoupling(Enum):
    AC = 0
    r'''
    AC coupling
    '''
    DC = 1
    r'''
    DC coupling
    '''
    HF_REJECT = 3
    r'''
    Highpass filter coupling
    '''
    LF_REJECT = 4
    r'''
    Lowpass filter coupling
    '''
    AC_PLUS_HF_REJECT = 1001
    r'''
    Highpass and lowpass filter coupling
    '''


class TriggerModifier(Enum):
    NO_TRIGGER_MOD = 1
    r'''
    Normal triggering.
    '''
    AUTO = 2
    r'''
    Software will trigger an acquisition automatically if no trigger arrives
    after a certain amount of time.
    '''
    AUTO_LEVEL = 3


class TriggerSlope(Enum):
    NEGATIVE = 0
    r'''
    Falling edge
    '''
    POSITIVE = 1
    r'''
    Rising edge
    '''
    SLOPE_EITHER = 3
    r'''
    Either edge
    '''


class TriggerType(Enum):
    EDGE = 1
    r'''
    Configures the digitizer for edge triggering.  An edge trigger occurs when the trigger signal crosses the trigger level specified with the set trigger slope.  You configure the trigger level and slope with configure_trigger_edge.
    '''
    HYSTERESIS = 1001
    r'''
    Configures the digitizer for hysteresis triggering.  A hysteresis trigger occurs when the trigger signal crosses the trigger level with the specified slope and passes through the hysteresis window you specify. You configure the trigger level, slope, and hysteresis with configure_trigger_hysteresis.
    '''
    DIGITAL = 1002
    r'''
    Configures the digitizer for digital triggering. A digital trigger occurs when the trigger signal has the specified slope. You configure the trigger slope with configure_trigger_digital.
    '''
    WINDOW = 1003
    r'''
    Configures the digitizer for window triggering.  A window trigger occurs when the trigger signal enters or leaves the window defined by the values you specify with the Low Window Level, High Window Level, and Window Mode Parameters.  You configure the low window level high window level, and window mode with configure_trigger_window.
    '''
    SOFTWARE = 1004
    r'''
    Configures the digitizer for software triggering.  A software trigger occurs when SendSoftwareTrigger is called.
    '''
    TV = 5
    r'''
    Configures the digitizer for video/TV triggering.   You configure the video trigger parameters like signal Format, Line to trigger off of, Polarity, and Enable DC Restore with configure_trigger_video.
    '''
    GLITCH = 4
    WIDTH = 2
    RUNT = 3
    IMMEDIATE = 6
    r'''
    Configures the digitizer for immediate triggering.   An immediate trigger occurs as soon as the pretrigger samples are acquired.
    '''


class TriggerWindowMode(Enum):
    ENTERING = 0
    r'''
    Trigger upon entering the window
    '''
    LEAVING = 1
    r'''
    Trigger upon leaving the window
    '''
    ENTERING_OR_LEAVING = 2


class VerticalCoupling(Enum):
    AC = 0
    r'''
    AC coupling
    '''
    DC = 1
    r'''
    DC coupling
    '''
    GND = 2
    r'''
    GND coupling
    '''


class VideoPolarity(Enum):
    POSITIVE = 1
    r'''
    Specifies that the video signal has positive polarity.
    '''
    NEGATIVE = 2
    r'''
    Specifies that the video signal has negative polarity.
    '''


class VideoSignalFormat(Enum):
    NTSC = 1
    r'''
    NTSC signal format supports line numbers from 1 to 525
    '''
    PAL = 2
    r'''
    PAL signal format supports line numbers from 1 to 625
    '''
    SECAM = 3
    r'''
    SECAM signal format supports line numbers from 1 to 625
    '''
    M_PAL = 1001
    r'''
    M-PAL signal format supports line numbers from 1 to 525
    '''
    VIDEO_480I_59_94_FIELDS_PER_SECOND = 1010
    r'''
    480 lines, interlaced, 59.94 fields per second
    '''
    VIDEO_480I_60_FIELDS_PER_SECOND = 1011
    r'''
    480 lines, interlaced, 60 fields per second
    '''
    VIDEO_480P_59_94_FRAMES_PER_SECOND = 1015
    r'''
    480 lines, progressive, 59.94 frames per second
    '''
    VIDEO_480P_60_FRAMES_PER_SECOND = 1016
    r'''
    480 lines, progressive,60 frames per second
    '''
    VIDEO_576I_50_FIELDS_PER_SECOND = 1020
    r'''
    576 lines, interlaced, 50 fields per second
    '''
    VIDEO_576P_50_FRAMES_PER_SECOND = 1025
    r'''
    576 lines, progressive, 50 frames per second
    '''
    VIDEO_720P_50_FRAMES_PER_SECOND = 1031
    r'''
    720 lines, progressive, 50 frames per second
    '''
    VIDEO_720P_59_94_FRAMES_PER_SECOND = 1032
    r'''
    720 lines, progressive, 59.94 frames per second
    '''
    VIDEO_720P_60_FRAMES_PER_SECOND = 1033
    r'''
    720 lines, progressive, 60 frames per second
    '''
    VIDEO_1080I_50_FIELDS_PER_SECOND = 1040
    r'''
    1,080 lines, interlaced, 50 fields per second
    '''
    VIDEO_1080I_59_94_FIELDS_PER_SECOND = 1041
    r'''
    1,080 lines, interlaced, 59.94 fields per second
    '''
    VIDEO_1080I_60_FIELDS_PER_SECOND = 1042
    r'''
    1,080 lines, interlaced, 60 fields per second
    '''
    VIDEO_1080P_24_FRAMES_PER_SECOND = 1045
    r'''
    1,080 lines, progressive, 24 frames per second
    '''


class VideoTriggerEvent(Enum):
    FIELD1 = 1
    r'''
    Trigger on field 1 of the signal
    '''
    FIELD2 = 2
    r'''
    Trigger on field 2 of the signal
    '''
    ANY_FIELD = 3
    r'''
    Trigger on the first field acquired
    '''
    ANY_LINE = 4
    r'''
    Trigger on the first line acquired
    '''
    LINE_NUMBER = 5
    r'''
    Trigger on a specific line of a video signal.  Valid values vary depending on the signal format configured.
    '''


class WhichTrigger(Enum):
    START = 0
    ARM_REFERENCE = 1
    REFERENCE = 2
    ADVANCE = 3


class WidthCondition(Enum):
    WITHIN = 1
    r'''
    Trigger on pulses with a duration within the range bounded by width_low_threshold and width_high_threshold.
    '''
    OUTSIDE = 2
    r'''
    Trigger on pulses with a duration not within the range bounded by width_low_threshold and width_high_threshold.
    '''


class WidthPolarity(Enum):
    POSITIVE = 1
    r'''
    Trigger on pulses of positive polarity relative to the trigger threshold.
    '''
    NEGATIVE = 2
    r'''
    Trigger on pulses of negative polarity relative to the trigger threshold.
    '''
    EITHER = 3
    r'''
    Trigger on pulses of either positive or negative polarity.
    '''



# -*- coding: utf-8 -*-
# This file was generated

import ctypes
import threading

from niscope._visatype import *  # noqa: F403,H303

import niscope.waveform_info as waveform_info  # noqa: F401

import niscope.measurement_stats as measurement_stats  # noqa: F401


class Library(object):
    '''Library

    Wrapper around driver library.
    Class will setup the correct ctypes information for every function on first call.
    '''

    def __init__(self, ctypes_library):
        self._func_lock = threading.Lock()
        self._library = ctypes_library
        # We cache the cfunc object from the ctypes.CDLL object
        self.niScope_Abort_cfunc = None
        self.niScope_AcquisitionStatus_cfunc = None
        self.niScope_ActualMeasWfmSize_cfunc = None
        self.niScope_ActualNumWfms_cfunc = None
        self.niScope_AddWaveformProcessing_cfunc = None
        self.niScope_AutoSetup_cfunc = None
        self.niScope_CalFetchDate_cfunc = None
        self.niScope_CalFetchTemperature_cfunc = None
        self.niScope_CalSelfCalibrate_cfunc = None
        self.niScope_ClearWaveformMeasurementStats_cfunc = None
        self.niScope_ClearWaveformProcessing_cfunc = None
        self.niScope_Commit_cfunc = None
        self.niScope_ConfigureChanCharacteristics_cfunc = None
        self.niScope_ConfigureEqualizationFilterCoefficients_cfunc = None
        self.niScope_ConfigureHorizontalTiming_cfunc = None
        self.niScope_ConfigureRefLevels_cfunc = None
        self.niScope_ConfigureTriggerDigital_cfunc = None
        self.niScope_ConfigureTriggerEdge_cfunc = None
        self.niScope_ConfigureTriggerHysteresis_cfunc = None
        self.niScope_ConfigureTriggerImmediate_cfunc = None
        self.niScope_ConfigureTriggerSoftware_cfunc = None
        self.niScope_ConfigureTriggerVideo_cfunc = None
        self.niScope_ConfigureTriggerWindow_cfunc = None
        self.niScope_ConfigureVertical_cfunc = None
        self.niScope_Disable_cfunc = None
        self.niScope_ExportAttributeConfigurationBuffer_cfunc = None
        self.niScope_ExportAttributeConfigurationFile_cfunc = None
        self.niScope_Fetch_cfunc = None
        self.niScope_FetchArrayMeasurement_cfunc = None
        self.niScope_FetchBinary16_cfunc = None
        self.niScope_FetchBinary32_cfunc = None
        self.niScope_FetchBinary8_cfunc = None
        self.niScope_FetchMeasurementStats_cfunc = None
        self.niScope_GetAttributeViBoolean_cfunc = None
        self.niScope_GetAttributeViInt32_cfunc = None
        self.niScope_GetAttributeViInt64_cfunc = None
        self.niScope_GetAttributeViReal64_cfunc = None
        self.niScope_GetAttributeViString_cfunc = None
        self.niScope_GetEqualizationFilterCoefficients_cfunc = None
        self.niScope_GetError_cfunc = None
        self.niScope_ImportAttributeConfigurationBuffer_cfunc = None
        self.niScope_ImportAttributeConfigurationFile_cfunc = None
        self.niScope_InitWithOptions_cfunc = None
        self.niScope_InitiateAcquisition_cfunc = None
        self.niScope_LockSession_cfunc = None
        self.niScope_ProbeCompensationSignalStart_cfunc = None
        self.niScope_ProbeCompensationSignalStop_cfunc = None
        self.niScope_Read_cfunc = None
        self.niScope_ResetDevice_cfunc = None
        self.niScope_ResetWithDefaults_cfunc = None
        self.niScope_SendSoftwareTriggerEdge_cfunc = None
        self.niScope_SetAttributeViBoolean_cfunc = None
        self.niScope_SetAttributeViInt32_cfunc = None
        self.niScope_SetAttributeViInt64_cfunc = None
        self.niScope_SetAttributeViReal64_cfunc = None
        self.niScope_SetAttributeViString_cfunc = None
        self.niScope_UnlockSession_cfunc = None
        self.niScope_close_cfunc = None
        self.niScope_error_message_cfunc = None
        self.niScope_reset_cfunc = None
        self.niScope_self_test_cfunc = None

    def niScope_Abort(self, vi):  # noqa: N802
        with self._func_lock:
            if self.niScope_Abort_cfunc is None:
                self.niScope_Abort_cfunc = self._library.niScope_Abort
                self.niScope_Abort_cfunc.argtypes = [ViSession]  # noqa: F405
                self.niScope_Abort_cfunc.restype = ViStatus  # noqa: F405
        return self.niScope_Abort_cfunc(vi)

    def niScope_AcquisitionStatus(self, vi, acquisition_status):  # noqa: N802
        with self._func_lock:
            if self.niScope_AcquisitionStatus_cfunc is None:
                self.niScope_AcquisitionStatus_cfunc = self._library.niScope_AcquisitionStatus
                self.niScope_AcquisitionStatus_cfunc.argtypes = [ViSession, ctypes.POINTER(ViInt32)]  # noqa: F405
                self.niScope_AcquisitionStatus_cfunc.restype = ViStatus  # noqa: F405
        return self.niScope_AcquisitionStatus_cfunc(vi, acquisition_status)

    def niScope_ActualMeasWfmSize(self, vi, array_meas_function, meas_waveform_size):  # noqa: N802
        with self._func_lock:
            if self.niScope_ActualMeasWfmSize_cfunc is None:
                self.niScope_ActualMeasWfmSize_cfunc = self._library.niScope_ActualMeasWfmSize
                self.niScope_ActualMeasWfmSize_cfunc.argtypes = [ViSession, ViInt32, ctypes.POINTER(ViInt32)]  # noqa: F405
                self.niScope_ActualMeasWfmSize_cfunc.restype = ViStatus  # noqa: F405
        return self.niScope_ActualMeasWfmSize_cfunc(vi, array_meas_function, meas_waveform_size)

    def niScope_ActualNumWfms(self, vi, channel_list, num_wfms):  # noqa: N802
        with self._func_lock:
            if self.niScope_ActualNumWfms_cfunc is None:
                self.niScope_ActualNumWfms_cfunc = self._library.niScope_ActualNumWfms
                self.niScope_ActualNumWfms_cfunc.argtypes = [ViSession, ctypes.POINTER(ViChar), ctypes.POINTER(ViInt32)]  # noqa: F405
                self.niScope_ActualNumWfms_cfunc.restype = ViStatus  # noqa: F405
        return self.niScope_ActualNumWfms_cfunc(vi, channel_list, num_wfms)

    def niScope_AddWaveformProcessing(self, vi, channel_list, meas_function):  # noqa: N802
        with self._func_lock:
            if self.niScope_AddWaveformProcessing_cfunc is None:
                self.niScope_AddWaveformProcessing_cfunc = self._library.niScope_AddWaveformProcessing
                self.niScope_AddWaveformProcessing_cfunc.argtypes = [ViSession, ctypes.POINTER(ViChar), ViInt32]  # noqa: F405
                self.niScope_AddWaveformProcessing_cfunc.restype = ViStatus  # noqa: F405
        return self.niScope_AddWaveformProcessing_cfunc(vi, channel_list, meas_function)

    def niScope_AutoSetup(self, vi):  # noqa: N802
        with self._func_lock:
            if self.niScope_AutoSetup_cfunc is None:
                self.niScope_AutoSetup_cfunc = self._library.niScope_AutoSetup
                self.niScope_AutoSetup_cfunc.argtypes = [ViSession]  # noqa: F405
                self.niScope_AutoSetup_cfunc.restype = ViStatus  # noqa: F405
        return self.niScope_AutoSetup_cfunc(vi)

    def niScope_CalFetchDate(self, vi, which_one, year, month, day):  # noqa: N802
        with self._func_lock:
            if self.niScope_CalFetchDate_cfunc is None:
                self.niScope_CalFetchDate_cfunc = self._library.niScope_CalFetchDate
                self.niScope_CalFetchDate_cfunc.argtypes = [ViSession, ViInt32, ctypes.POINTER(ViInt32), ctypes.POINTER(ViInt32), ctypes.POINTER(ViInt32)]  # noqa: F405
                self.niScope_CalFetchDate_cfunc.restype = ViStatus  # noqa: F405
        return self.niScope_CalFetchDate_cfunc(vi, which_one, year, month, day)

    def niScope_CalFetchTemperature(self, vi, which_one, temperature):  # noqa: N802
        with self._func_lock:
            if self.niScope_CalFetchTemperature_cfunc is None:
                self.niScope_CalFetchTemperature_cfunc = self._library.niScope_CalFetchTemperature
                self.niScope_CalFetchTemperature_cfunc.argtypes = [ViSession, ViInt32, ctypes.POINTER(ViReal64)]  # noqa: F405
                self.niScope_CalFetchTemperature_cfunc.restype = ViStatus  # noqa: F405
        return self.niScope_CalFetchTemperature_cfunc(vi, which_one, temperature)

    def niScope_CalSelfCalibrate(self, vi, channel_list, option):  # noqa: N802
        with self._func_lock:
            if self.niScope_CalSelfCalibrate_cfunc is None:
                self.niScope_CalSelfCalibrate_cfunc = self._library.niScope_CalSelfCalibrate
                self.niScope_CalSelfCalibrate_cfunc.argtypes = [ViSession, ctypes.POINTER(ViChar), ViInt32]  # noqa: F405
                self.niScope_CalSelfCalibrate_cfunc.restype = ViStatus  # noqa: F405
        return self.niScope_CalSelfCalibrate_cfunc(vi, channel_list, option)

    def niScope_ClearWaveformMeasurementStats(self, vi, channel_list, clearable_measurement_function):  # noqa: N802
        with self._func_lock:
            if self.niScope_ClearWaveformMeasurementStats_cfunc is None:
                self.niScope_ClearWaveformMeasurementStats_cfunc = self._library.niScope_ClearWaveformMeasurementStats
                self.niScope_ClearWaveformMeasurementStats_cfunc.argtypes = [ViSession, ctypes.POINTER(ViChar), ViInt32]  # noqa: F405
                self.niScope_ClearWaveformMeasurementStats_cfunc.restype = ViStatus  # noqa: F405
        return self.niScope_ClearWaveformMeasurementStats_cfunc(vi, channel_list, clearable_measurement_function)

    def niScope_ClearWaveformProcessing(self, vi, channel_list):  # noqa: N802
        with self._func_lock:
            if self.niScope_ClearWaveformProcessing_cfunc is None:
                self.niScope_ClearWaveformProcessing_cfunc = self._library.niScope_ClearWaveformProcessing
                self.niScope_ClearWaveformProcessing_cfunc.argtypes = [ViSession, ctypes.POINTER(ViChar)]  # noqa: F405
                self.niScope_ClearWaveformProcessing_cfunc.restype = ViStatus  # noqa: F405
        return self.niScope_ClearWaveformProcessing_cfunc(vi, channel_list)

    def niScope_Commit(self, vi):  # noqa: N802
        with self._func_lock:
            if self.niScope_Commit_cfunc is None:
                self.niScope_Commit_cfunc = self._library.niScope_Commit
                self.niScope_Commit_cfunc.argtypes = [ViSession]  # noqa: F405
                self.niScope_Commit_cfunc.restype = ViStatus  # noqa: F405
        return self.niScope_Commit_cfunc(vi)

    def niScope_ConfigureChanCharacteristics(self, vi, channel_list, input_impedance, max_input_frequency):  # noqa: N802
        with self._func_lock:
            if self.niScope_ConfigureChanCharacteristics_cfunc is None:
                self.niScope_ConfigureChanCharacteristics_cfunc = self._library.niScope_ConfigureChanCharacteristics
                self.niScope_ConfigureChanCharacteristics_cfunc.argtypes = [ViSession, ctypes.POINTER(ViChar), ViReal64, ViReal64]  # noqa: F405
                self.niScope_ConfigureChanCharacteristics_cfunc.restype = ViStatus  # noqa: F405
        return self.niScope_ConfigureChanCharacteristics_cfunc(vi, channel_list, input_impedance, max_input_frequency)

    def niScope_ConfigureEqualizationFilterCoefficients(self, vi, channel_list, number_of_coefficients, coefficients):  # noqa: N802
        with self._func_lock:
            if self.niScope_ConfigureEqualizationFilterCoefficients_cfunc is None:
                self.niScope_ConfigureEqualizationFilterCoefficients_cfunc = self._library.niScope_ConfigureEqualizationFilterCoefficients
                self.niScope_ConfigureEqualizationFilterCoefficients_cfunc.argtypes = [ViSession, ctypes.POINTER(ViChar), ViInt32, ctypes.POINTER(ViReal64)]  # noqa: F405
                self.niScope_ConfigureEqualizationFilterCoefficients_cfunc.restype = ViStatus  # noqa: F405
        return self.niScope_ConfigureEqualizationFilterCoefficients_cfunc(vi, channel_list, number_of_coefficients, coefficients)

    def niScope_ConfigureHorizontalTiming(self, vi, min_sample_rate, min_num_pts, ref_position, num_records, enforce_realtime):  # noqa: N802
        with self._func_lock:
            if self.niScope_ConfigureHorizontalTiming_cfunc is None:
                self.niScope_ConfigureHorizontalTiming_cfunc = self._library.niScope_ConfigureHorizontalTiming
                self.niScope_ConfigureHorizontalTiming_cfunc.argtypes = [ViSession, ViReal64, ViInt32, ViReal64, ViInt32, ViBoolean]  # noqa: F405
                self.niScope_ConfigureHorizontalTiming_cfunc.restype = ViStatus  # noqa: F405
        return self.niScope_ConfigureHorizontalTiming_cfunc(vi, min_sample_rate, min_num_pts, ref_position, num_records, enforce_realtime)

    def niScope_ConfigureRefLevels(self, vi, low, mid, high):  # noqa: N802
        with self._func_lock:
            if self.niScope_ConfigureRefLevels_cfunc is None:
                self.niScope_ConfigureRefLevels_cfunc = self._library.niScope_ConfigureRefLevels
                self.niScope_ConfigureRefLevels_cfunc.argtypes = [ViSession, ViReal64, ViReal64, ViReal64]  # noqa: F405
                self.niScope_ConfigureRefLevels_cfunc.restype = ViStatus  # noqa: F405
        return self.niScope_ConfigureRefLevels_cfunc(vi, low, mid, high)

    def niScope_ConfigureTriggerDigital(self, vi, trigger_source, slope, holdoff, delay):  # noqa: N802
        with self._func_lock:
            if self.niScope_ConfigureTriggerDigital_cfunc is None:
                self.niScope_ConfigureTriggerDigital_cfunc = self._library.niScope_ConfigureTriggerDigital
                self.niScope_ConfigureTriggerDigital_cfunc.argtypes = [ViSession, ctypes.POINTER(ViChar), ViInt32, ViReal64, ViReal64]  # noqa: F405
                self.niScope_ConfigureTriggerDigital_cfunc.restype = ViStatus  # noqa: F405
        return self.niScope_ConfigureTriggerDigital_cfunc(vi, trigger_source, slope, holdoff, delay)

    def niScope_ConfigureTriggerEdge(self, vi, trigger_source, level, slope, trigger_coupling, holdoff, delay):  # noqa: N802
        with self._func_lock:
            if self.niScope_ConfigureTriggerEdge_cfunc is None:
                self.niScope_ConfigureTriggerEdge_cfunc = self._library.niScope_ConfigureTriggerEdge
                self.niScope_ConfigureTriggerEdge_cfunc.argtypes = [ViSession, ctypes.POINTER(ViChar), ViReal64, ViInt32, ViInt32, ViReal64, ViReal64]  # noqa: F405
                self.niScope_ConfigureTriggerEdge_cfunc.restype = ViStatus  # noqa: F405
        return self.niScope_ConfigureTriggerEdge_cfunc(vi, trigger_source, level, slope, trigger_coupling, holdoff, delay)

    def niScope_ConfigureTriggerHysteresis(self, vi, trigger_source, level, hysteresis, slope, trigger_coupling, holdoff, delay):  # noqa: N802
        with self._func_lock:
            if self.niScope_ConfigureTriggerHysteresis_cfunc is None:
                self.niScope_ConfigureTriggerHysteresis_cfunc = self._library.niScope_ConfigureTriggerHysteresis
                self.niScope_ConfigureTriggerHysteresis_cfunc.argtypes = [ViSession, ctypes.POINTER(ViChar), ViReal64, ViReal64, ViInt32, ViInt32, ViReal64, ViReal64]  # noqa: F405
                self.niScope_ConfigureTriggerHysteresis_cfunc.restype = ViStatus  # noqa: F405
        return self.niScope_ConfigureTriggerHysteresis_cfunc(vi, trigger_source, level, hysteresis, slope, trigger_coupling, holdoff, delay)

    def niScope_ConfigureTriggerImmediate(self, vi):  # noqa: N802
        with self._func_lock:
            if self.niScope_ConfigureTriggerImmediate_cfunc is None:
                self.niScope_ConfigureTriggerImmediate_cfunc = self._library.niScope_ConfigureTriggerImmediate
                self.niScope_ConfigureTriggerImmediate_cfunc.argtypes = [ViSession]  # noqa: F405
                self.niScope_ConfigureTriggerImmediate_cfunc.restype = ViStatus  # noqa: F405
        return self.niScope_ConfigureTriggerImmediate_cfunc(vi)

    def niScope_ConfigureTriggerSoftware(self, vi, holdoff, delay):  # noqa: N802
        with self._func_lock:
            if self.niScope_ConfigureTriggerSoftware_cfunc is None:
                self.niScope_ConfigureTriggerSoftware_cfunc = self._library.niScope_ConfigureTriggerSoftware
                self.niScope_ConfigureTriggerSoftware_cfunc.argtypes = [ViSession, ViReal64, ViReal64]  # noqa: F405
                self.niScope_ConfigureTriggerSoftware_cfunc.restype = ViStatus  # noqa: F405
        return self.niScope_ConfigureTriggerSoftware_cfunc(vi, holdoff, delay)

    def niScope_ConfigureTriggerVideo(self, vi, trigger_source, enable_dc_restore, signal_format, event, line_number, polarity, trigger_coupling, holdoff, delay):  # noqa: N802
        with self._func_lock:
            if self.niScope_ConfigureTriggerVideo_cfunc is None:
                self.niScope_ConfigureTriggerVideo_cfunc = self._library.niScope_ConfigureTriggerVideo
                self.niScope_ConfigureTriggerVideo_cfunc.argtypes = [ViSession, ctypes.POINTER(ViChar), ViBoolean, ViInt32, ViInt32, ViInt32, ViInt32, ViInt32, ViReal64, ViReal64]  # noqa: F405
                self.niScope_ConfigureTriggerVideo_cfunc.restype = ViStatus  # noqa: F405
        return self.niScope_ConfigureTriggerVideo_cfunc(vi, trigger_source, enable_dc_restore, signal_format, event, line_number, polarity, trigger_coupling, holdoff, delay)

    def niScope_ConfigureTriggerWindow(self, vi, trigger_source, low_level, high_level, window_mode, trigger_coupling, holdoff, delay):  # noqa: N802
        with self._func_lock:
            if self.niScope_ConfigureTriggerWindow_cfunc is None:
                self.niScope_ConfigureTriggerWindow_cfunc = self._library.niScope_ConfigureTriggerWindow
                self.niScope_ConfigureTriggerWindow_cfunc.argtypes = [ViSession, ctypes.POINTER(ViChar), ViReal64, ViReal64, ViInt32, ViInt32, ViReal64, ViReal64]  # noqa: F405
                self.niScope_ConfigureTriggerWindow_cfunc.restype = ViStatus  # noqa: F405
        return self.niScope_ConfigureTriggerWindow_cfunc(vi, trigger_source, low_level, high_level, window_mode, trigger_coupling, holdoff, delay)

    def niScope_ConfigureVertical(self, vi, channel_list, range, offset, coupling, probe_attenuation, enabled):  # noqa: N802
        with self._func_lock:
            if self.niScope_ConfigureVertical_cfunc is None:
                self.niScope_ConfigureVertical_cfunc = self._library.niScope_ConfigureVertical
                self.niScope_ConfigureVertical_cfunc.argtypes = [ViSession, ctypes.POINTER(ViChar), ViReal64, ViReal64, ViInt32, ViReal64, ViBoolean]  # noqa: F405
                self.niScope_ConfigureVertical_cfunc.restype = ViStatus  # noqa: F405
        return self.niScope_ConfigureVertical_cfunc(vi, channel_list, range, offset, coupling, probe_attenuation, enabled)

    def niScope_Disable(self, vi):  # noqa: N802
        with self._func_lock:
            if self.niScope_Disable_cfunc is None:
                self.niScope_Disable_cfunc = self._library.niScope_Disable
                self.niScope_Disable_cfunc.argtypes = [ViSession]  # noqa: F405
                self.niScope_Disable_cfunc.restype = ViStatus  # noqa: F405
        return self.niScope_Disable_cfunc(vi)

    def niScope_ExportAttributeConfigurationBuffer(self, vi, size_in_bytes, configuration):  # noqa: N802
        with self._func_lock:
            if self.niScope_ExportAttributeConfigurationBuffer_cfunc is None:
                self.niScope_ExportAttributeConfigurationBuffer_cfunc = self._library.niScope_ExportAttributeConfigurationBuffer
                self.niScope_ExportAttributeConfigurationBuffer_cfunc.argtypes = [ViSession, ViInt32, ctypes.POINTER(ViInt8)]  # noqa: F405
                self.niScope_ExportAttributeConfigurationBuffer_cfunc.restype = ViStatus  # noqa: F405
        return self.niScope_ExportAttributeConfigurationBuffer_cfunc(vi, size_in_bytes, configuration)

    def niScope_ExportAttributeConfigurationFile(self, vi, file_path):  # noqa: N802
        with self._func_lock:
            if self.niScope_ExportAttributeConfigurationFile_cfunc is None:
                self.niScope_ExportAttributeConfigurationFile_cfunc = self._library.niScope_ExportAttributeConfigurationFile
                self.niScope_ExportAttributeConfigurationFile_cfunc.argtypes = [ViSession, ctypes.POINTER(ViChar)]  # noqa: F405
                self.niScope_ExportAttributeConfigurationFile_cfunc.restype = ViStatus  # noqa: F405
        return self.niScope_ExportAttributeConfigurationFile_cfunc(vi, file_path)

    def niScope_Fetch(self, vi, channel_list, timeout, num_samples, waveform, wfm_info):  # noqa: N802
        with self._func_lock:
            if self.niScope_Fetch_cfunc is None:
                self.niScope_Fetch_cfunc = self._library.niScope_Fetch
                self.niScope_Fetch_cfunc.argtypes = [ViSession, ctypes.POINTER(ViChar), ViReal64, ViInt32, ctypes.POINTER(ViReal64), ctypes.POINTER(waveform_info.struct_niScope_wfmInfo)]  # noqa: F405
                self.niScope_Fetch_cfunc.restype = ViStatus  # noqa: F405
        return self.niScope_Fetch_cfunc(vi, channel_list, timeout, num_samples, waveform, wfm_info)

    def niScope_FetchArrayMeasurement(self, vi, channel_list, timeout, array_meas_function, measurement_waveform_size, meas_wfm, wfm_info):  # noqa: N802
        with self._func_lock:
            if self.niScope_FetchArrayMeasurement_cfunc is None:
                self.niScope_FetchArrayMeasurement_cfunc = self._library.niScope_FetchArrayMeasurement
                self.niScope_FetchArrayMeasurement_cfunc.argtypes = [ViSession, ctypes.POINTER(ViChar), ViReal64, ViInt32, ViInt32, ctypes.POINTER(ViReal64), ctypes.POINTER(waveform_info.struct_niScope_wfmInfo)]  # noqa: F405
                self.niScope_FetchArrayMeasurement_cfunc.restype = ViStatus  # noqa: F405
        return self.niScope_FetchArrayMeasurement_cfunc(vi, channel_list, timeout, array_meas_function, measurement_waveform_size, meas_wfm, wfm_info)

    def niScope_FetchBinary16(self, vi, channel_list, timeout, num_samples, waveform, wfm_info):  # noqa: N802
        with self._func_lock:
            if self.niScope_FetchBinary16_cfunc is None:
                self.niScope_FetchBinary16_cfunc = self._library.niScope_FetchBinary16
                self.niScope_FetchBinary16_cfunc.argtypes = [ViSession, ctypes.POINTER(ViChar), ViReal64, ViInt32, ctypes.POINTER(ViInt16), ctypes.POINTER(waveform_info.struct_niScope_wfmInfo)]  # noqa: F405
                self.niScope_FetchBinary16_cfunc.restype = ViStatus  # noqa: F405
        return self.niScope_FetchBinary16_cfunc(vi, channel_list, timeout, num_samples, waveform, wfm_info)

    def niScope_FetchBinary32(self, vi, channel_list, timeout, num_samples, waveform, wfm_info):  # noqa: N802
        with self._func_lock:
            if self.niScope_FetchBinary32_cfunc is None:
                self.niScope_FetchBinary32_cfunc = self._library.niScope_FetchBinary32
                self.niScope_FetchBinary32_cfunc.argtypes = [ViSession, ctypes.POINTER(ViChar), ViReal64, ViInt32, ctypes.POINTER(ViInt32), ctypes.POINTER(waveform_info.struct_niScope_wfmInfo)]  # noqa: F405
                self.niScope_FetchBinary32_cfunc.restype = ViStatus  # noqa: F405
        return self.niScope_FetchBinary32_cfunc(vi, channel_list, timeout, num_samples, waveform, wfm_info)

    def niScope_FetchBinary8(self, vi, channel_list, timeout, num_samples, waveform, wfm_info):  # noqa: N802
        with self._func_lock:
            if self.niScope_FetchBinary8_cfunc is None:
                self.niScope_FetchBinary8_cfunc = self._library.niScope_FetchBinary8
                self.niScope_FetchBinary8_cfunc.argtypes = [ViSession, ctypes.POINTER(ViChar), ViReal64, ViInt32, ctypes.POINTER(ViInt8), ctypes.POINTER(waveform_info.struct_niScope_wfmInfo)]  # noqa: F405
                self.niScope_FetchBinary8_cfunc.restype = ViStatus  # noqa: F405
        return self.niScope_FetchBinary8_cfunc(vi, channel_list, timeout, num_samples, waveform, wfm_info)

    def niScope_FetchMeasurementStats(self, vi, channel_list, timeout, scalar_meas_function, result, mean, stdev, min, max, num_in_stats):  # noqa: N802
        with self._func_lock:
            if self.niScope_FetchMeasurementStats_cfunc is None:
                self.niScope_FetchMeasurementStats_cfunc = self._library.niScope_FetchMeasurementStats
                self.niScope_FetchMeasurementStats_cfunc.argtypes = [ViSession, ctypes.POINTER(ViChar), ViReal64, ViInt32, ctypes.POINTER(ViReal64), ctypes.POINTER(ViReal64), ctypes.POINTER(ViReal64), ctypes.POINTER(ViReal64), ctypes.POINTER(ViReal64), ctypes.POINTER(ViInt32)]  # noqa: F405
                self.niScope_FetchMeasurementStats_cfunc.restype = ViStatus  # noqa: F405
        return self.niScope_FetchMeasurementStats_cfunc(vi, channel_list, timeout, scalar_meas_function, result, mean, stdev, min, max, num_in_stats)

    def niScope_GetAttributeViBoolean(self, vi, channel_list, attribute_id, value):  # noqa: N802
        with self._func_lock:
            if self.niScope_GetAttributeViBoolean_cfunc is None:
                self.niScope_GetAttributeViBoolean_cfunc = self._library.niScope_GetAttributeViBoolean
                self.niScope_GetAttributeViBoolean_cfunc.argtypes = [ViSession, ctypes.POINTER(ViChar), ViAttr, ctypes.POINTER(ViBoolean)]  # noqa: F405
                self.niScope_GetAttributeViBoolean_cfunc.restype = ViStatus  # noqa: F405
        return self.niScope_GetAttributeViBoolean_cfunc(vi, channel_list, attribute_id, value)

    def niScope_GetAttributeViInt32(self, vi, channel_list, attribute_id, value):  # noqa: N802
        with self._func_lock:
            if self.niScope_GetAttributeViInt32_cfunc is None:
                self.niScope_GetAttributeViInt32_cfunc = self._library.niScope_GetAttributeViInt32
                self.niScope_GetAttributeViInt32_cfunc.argtypes = [ViSession, ctypes.POINTER(ViChar), ViAttr, ctypes.POINTER(ViInt32)]  # noqa: F405
                self.niScope_GetAttributeViInt32_cfunc.restype = ViStatus  # noqa: F405
        return self.niScope_GetAttributeViInt32_cfunc(vi, channel_list, attribute_id, value)

    def niScope_GetAttributeViInt64(self, vi, channel_list, attribute_id, value):  # noqa: N802
        with self._func_lock:
            if self.niScope_GetAttributeViInt64_cfunc is None:
                self.niScope_GetAttributeViInt64_cfunc = self._library.niScope_GetAttributeViInt64
                self.niScope_GetAttributeViInt64_cfunc.argtypes = [ViSession, ctypes.POINTER(ViChar), ViAttr, ctypes.POINTER(ViInt64)]  # noqa: F405
                self.niScope_GetAttributeViInt64_cfunc.restype = ViStatus  # noqa: F405
        return self.niScope_GetAttributeViInt64_cfunc(vi, channel_list, attribute_id, value)

    def niScope_GetAttributeViReal64(self, vi, channel_list, attribute_id, value):  # noqa: N802
        with self._func_lock:
            if self.niScope_GetAttributeViReal64_cfunc is None:
                self.niScope_GetAttributeViReal64_cfunc = self._library.niScope_GetAttributeViReal64
                self.niScope_GetAttributeViReal64_cfunc.argtypes = [ViSession, ctypes.POINTER(ViChar), ViAttr, ctypes.POINTER(ViReal64)]  # noqa: F405
                self.niScope_GetAttributeViReal64_cfunc.restype = ViStatus  # noqa: F405
        return self.niScope_GetAttributeViReal64_cfunc(vi, channel_list, attribute_id, value)

    def niScope_GetAttributeViString(self, vi, channel_list, attribute_id, buf_size, value):  # noqa: N802
        with self._func_lock:
            if self.niScope_GetAttributeViString_cfunc is None:
                self.niScope_GetAttributeViString_cfunc = self._library.niScope_GetAttributeViString
                self.niScope_GetAttributeViString_cfunc.argtypes = [ViSession, ctypes.POINTER(ViChar), ViAttr, ViInt32, ctypes.POINTER(ViChar)]  # noqa: F405
                self.niScope_GetAttributeViString_cfunc.restype = ViStatus  # noqa: F405
        return self.niScope_GetAttributeViString_cfunc(vi, channel_list, attribute_id, buf_size, value)

    def niScope_GetEqualizationFilterCoefficients(self, vi, channel, number_of_coefficients, coefficients):  # noqa: N802
        with self._func_lock:
            if self.niScope_GetEqualizationFilterCoefficients_cfunc is None:
                self.niScope_GetEqualizationFilterCoefficients_cfunc = self._library.niScope_GetEqualizationFilterCoefficients
                self.niScope_GetEqualizationFilterCoefficients_cfunc.argtypes = [ViSession, ctypes.POINTER(ViChar), ViInt32, ctypes.POINTER(ViReal64)]  # noqa: F405
                self.niScope_GetEqualizationFilterCoefficients_cfunc.restype = ViStatus  # noqa: F405
        return self.niScope_GetEqualizationFilterCoefficients_cfunc(vi, channel, number_of_coefficients, coefficients)

    def niScope_GetError(self, vi, error_code, buffer_size, description):  # noqa: N802
        with self._func_lock:
            if self.niScope_GetError_cfunc is None:
                self.niScope_GetError_cfunc = self._library.niScope_GetError
                self.niScope_GetError_cfunc.argtypes = [ViSession, ctypes.POINTER(ViStatus), ViInt32, ctypes.POINTER(ViChar)]  # noqa: F405
                self.niScope_GetError_cfunc.restype = ViStatus  # noqa: F405
        return self.niScope_GetError_cfunc(vi, error_code, buffer_size, description)

    def niScope_ImportAttributeConfigurationBuffer(self, vi, size_in_bytes, configuration):  # noqa: N802
        with self._func_lock:
            if self.niScope_ImportAttributeConfigurationBuffer_cfunc is None:
                self.niScope_ImportAttributeConfigurationBuffer_cfunc = self._library.niScope_ImportAttributeConfigurationBuffer
                self.niScope_ImportAttributeConfigurationBuffer_cfunc.argtypes = [ViSession, ViInt32, ctypes.POINTER(ViInt8)]  # noqa: F405
                self.niScope_ImportAttributeConfigurationBuffer_cfunc.restype = ViStatus  # noqa: F405
        return self.niScope_ImportAttributeConfigurationBuffer_cfunc(vi, size_in_bytes, configuration)

    def niScope_ImportAttributeConfigurationFile(self, vi, file_path):  # noqa: N802
        with self._func_lock:
            if self.niScope_ImportAttributeConfigurationFile_cfunc is None:
                self.niScope_ImportAttributeConfigurationFile_cfunc = self._library.niScope_ImportAttributeConfigurationFile
                self.niScope_ImportAttributeConfigurationFile_cfunc.argtypes = [ViSession, ctypes.POINTER(ViChar)]  # noqa: F405
                self.niScope_ImportAttributeConfigurationFile_cfunc.restype = ViStatus  # noqa: F405
        return self.niScope_ImportAttributeConfigurationFile_cfunc(vi, file_path)

    def niScope_InitWithOptions(self, resource_name, id_query, reset_device, option_string, vi):  # noqa: N802
        with self._func_lock:
            if self.niScope_InitWithOptions_cfunc is None:
                self.niScope_InitWithOptions_cfunc = self._library.niScope_InitWithOptions
                self.niScope_InitWithOptions_cfunc.argtypes = [ctypes.POINTER(ViChar), ViBoolean, ViBoolean, ctypes.POINTER(ViChar), ctypes.POINTER(ViSession)]  # noqa: F405
                self.niScope_InitWithOptions_cfunc.restype = ViStatus  # noqa: F405
        return self.niScope_InitWithOptions_cfunc(resource_name, id_query, reset_device, option_string, vi)

    def niScope_InitiateAcquisition(self, vi):  # noqa: N802
        with self._func_lock:
            if self.niScope_InitiateAcquisition_cfunc is None:
                self.niScope_InitiateAcquisition_cfunc = self._library.niScope_InitiateAcquisition
                self.niScope_InitiateAcquisition_cfunc.argtypes = [ViSession]  # noqa: F405
                self.niScope_InitiateAcquisition_cfunc.restype = ViStatus  # noqa: F405
        return self.niScope_InitiateAcquisition_cfunc(vi)

    def niScope_LockSession(self, vi, caller_has_lock):  # noqa: N802
        with self._func_lock:
            if self.niScope_LockSession_cfunc is None:
                self.niScope_LockSession_cfunc = self._library.niScope_LockSession
                self.niScope_LockSession_cfunc.argtypes = [ViSession, ctypes.POINTER(ViBoolean)]  # noqa: F405
                self.niScope_LockSession_cfunc.restype = ViStatus  # noqa: F405
        return self.niScope_LockSession_cfunc(vi, caller_has_lock)

    def niScope_ProbeCompensationSignalStart(self, vi):  # noqa: N802
        with self._func_lock:
            if self.niScope_ProbeCompensationSignalStart_cfunc is None:
                self.niScope_ProbeCompensationSignalStart_cfunc = self._library.niScope_ProbeCompensationSignalStart
                self.niScope_ProbeCompensationSignalStart_cfunc.argtypes = [ViSession]  # noqa: F405
                self.niScope_ProbeCompensationSignalStart_cfunc.restype = ViStatus  # noqa: F405
        return self.niScope_ProbeCompensationSignalStart_cfunc(vi)

    def niScope_ProbeCompensationSignalStop(self, vi):  # noqa: N802
        with self._func_lock:
            if self.niScope_ProbeCompensationSignalStop_cfunc is None:
                self.niScope_ProbeCompensationSignalStop_cfunc = self._library.niScope_ProbeCompensationSignalStop
                self.niScope_ProbeCompensationSignalStop_cfunc.argtypes = [ViSession]  # noqa: F405
                self.niScope_ProbeCompensationSignalStop_cfunc.restype = ViStatus  # noqa: F405
        return self.niScope_ProbeCompensationSignalStop_cfunc(vi)

    def niScope_Read(self, vi, channel_list, timeout, num_samples, waveform, wfm_info):  # noqa: N802
        with self._func_lock:
            if self.niScope_Read_cfunc is None:
                self.niScope_Read_cfunc = self._library.niScope_Read
                self.niScope_Read_cfunc.argtypes = [ViSession, ctypes.POINTER(ViChar), ViReal64, ViInt32, ctypes.POINTER(ViReal64), ctypes.POINTER(waveform_info.struct_niScope_wfmInfo)]  # noqa: F405
                self.niScope_Read_cfunc.restype = ViStatus  # noqa: F405
        return self.niScope_Read_cfunc(vi, channel_list, timeout, num_samples, waveform, wfm_info)

    def niScope_ResetDevice(self, vi):  # noqa: N802
        with self._func_lock:
            if self.niScope_ResetDevice_cfunc is None:
                self.niScope_ResetDevice_cfunc = self._library.niScope_ResetDevice
                self.niScope_ResetDevice_cfunc.argtypes = [ViSession]  # noqa: F405
                self.niScope_ResetDevice_cfunc.restype = ViStatus  # noqa: F405
        return self.niScope_ResetDevice_cfunc(vi)

    def niScope_ResetWithDefaults(self, vi):  # noqa: N802
        with self._func_lock:
            if self.niScope_ResetWithDefaults_cfunc is None:
                self.niScope_ResetWithDefaults_cfunc = self._library.niScope_ResetWithDefaults
                self.niScope_ResetWithDefaults_cfunc.argtypes = [ViSession]  # noqa: F405
                self.niScope_ResetWithDefaults_cfunc.restype = ViStatus  # noqa: F405
        return self.niScope_ResetWithDefaults_cfunc(vi)

    def niScope_SendSoftwareTriggerEdge(self, vi, which_trigger):  # noqa: N802
        with self._func_lock:
            if self.niScope_SendSoftwareTriggerEdge_cfunc is None:
                self.niScope_SendSoftwareTriggerEdge_cfunc = self._library.niScope_SendSoftwareTriggerEdge
                self.niScope_SendSoftwareTriggerEdge_cfunc.argtypes = [ViSession, ViInt32]  # noqa: F405
                self.niScope_SendSoftwareTriggerEdge_cfunc.restype = ViStatus  # noqa: F405
        return self.niScope_SendSoftwareTriggerEdge_cfunc(vi, which_trigger)

    def niScope_SetAttributeViBoolean(self, vi, channel_list, attribute_id, value):  # noqa: N802
        with self._func_lock:
            if self.niScope_SetAttributeViBoolean_cfunc is None:
                self.niScope_SetAttributeViBoolean_cfunc = self._library.niScope_SetAttributeViBoolean
                self.niScope_SetAttributeViBoolean_cfunc.argtypes = [ViSession, ctypes.POINTER(ViChar), ViAttr, ViBoolean]  # noqa: F405
                self.niScope_SetAttributeViBoolean_cfunc.restype = ViStatus  # noqa: F405
        return self.niScope_SetAttributeViBoolean_cfunc(vi, channel_list, attribute_id, value)

    def niScope_SetAttributeViInt32(self, vi, channel_list, attribute_id, value):  # noqa: N802
        with self._func_lock:
            if self.niScope_SetAttributeViInt32_cfunc is None:
                self.niScope_SetAttributeViInt32_cfunc = self._library.niScope_SetAttributeViInt32
                self.niScope_SetAttributeViInt32_cfunc.argtypes = [ViSession, ctypes.POINTER(ViChar), ViAttr, ViInt32]  # noqa: F405
                self.niScope_SetAttributeViInt32_cfunc.restype = ViStatus  # noqa: F405
        return self.niScope_SetAttributeViInt32_cfunc(vi, channel_list, attribute_id, value)

    def niScope_SetAttributeViInt64(self, vi, channel_list, attribute_id, value):  # noqa: N802
        with self._func_lock:
            if self.niScope_SetAttributeViInt64_cfunc is None:
                self.niScope_SetAttributeViInt64_cfunc = self._library.niScope_SetAttributeViInt64
                self.niScope_SetAttributeViInt64_cfunc.argtypes = [ViSession, ctypes.POINTER(ViChar), ViAttr, ViInt64]  # noqa: F405
                self.niScope_SetAttributeViInt64_cfunc.restype = ViStatus  # noqa: F405
        return self.niScope_SetAttributeViInt64_cfunc(vi, channel_list, attribute_id, value)

    def niScope_SetAttributeViReal64(self, vi, channel_list, attribute_id, value):  # noqa: N802
        with self._func_lock:
            if self.niScope_SetAttributeViReal64_cfunc is None:
                self.niScope_SetAttributeViReal64_cfunc = self._library.niScope_SetAttributeViReal64
                self.niScope_SetAttributeViReal64_cfunc.argtypes = [ViSession, ctypes.POINTER(ViChar), ViAttr, ViReal64]  # noqa: F405
                self.niScope_SetAttributeViReal64_cfunc.restype = ViStatus  # noqa: F405
        return self.niScope_SetAttributeViReal64_cfunc(vi, channel_list, attribute_id, value)

    def niScope_SetAttributeViString(self, vi, channel_list, attribute_id, value):  # noqa: N802
        with self._func_lock:
            if self.niScope_SetAttributeViString_cfunc is None:
                self.niScope_SetAttributeViString_cfunc = self._library.niScope_SetAttributeViString
                self.niScope_SetAttributeViString_cfunc.argtypes = [ViSession, ctypes.POINTER(ViChar), ViAttr, ctypes.POINTER(ViChar)]  # noqa: F405
                self.niScope_SetAttributeViString_cfunc.restype = ViStatus  # noqa: F405
        return self.niScope_SetAttributeViString_cfunc(vi, channel_list, attribute_id, value)

    def niScope_UnlockSession(self, vi, caller_has_lock):  # noqa: N802
        with self._func_lock:
            if self.niScope_UnlockSession_cfunc is None:
                self.niScope_UnlockSession_cfunc = self._library.niScope_UnlockSession
                self.niScope_UnlockSession_cfunc.argtypes = [ViSession, ctypes.POINTER(ViBoolean)]  # noqa: F405
                self.niScope_UnlockSession_cfunc.restype = ViStatus  # noqa: F405
        return self.niScope_UnlockSession_cfunc(vi, caller_has_lock)

    def niScope_close(self, vi):  # noqa: N802
        with self._func_lock:
            if self.niScope_close_cfunc is None:
                self.niScope_close_cfunc = self._library.niScope_close
                self.niScope_close_cfunc.argtypes = [ViSession]  # noqa: F405
                self.niScope_close_cfunc.restype = ViStatus  # noqa: F405
        return self.niScope_close_cfunc(vi)

    def niScope_error_message(self, vi, error_code, error_message):  # noqa: N802
        with self._func_lock:
            if self.niScope_error_message_cfunc is None:
                self.niScope_error_message_cfunc = self._library.niScope_error_message
                self.niScope_error_message_cfunc.argtypes = [ViSession, ViStatus, ctypes.POINTER(ViChar)]  # noqa: F405
                self.niScope_error_message_cfunc.restype = ViStatus  # noqa: F405
        return self.niScope_error_message_cfunc(vi, error_code, error_message)

    def niScope_reset(self, vi):  # noqa: N802
        with self._func_lock:
            if self.niScope_reset_cfunc is None:
                self.niScope_reset_cfunc = self._library.niScope_reset
                self.niScope_reset_cfunc.argtypes = [ViSession]  # noqa: F405
                self.niScope_reset_cfunc.restype = ViStatus  # noqa: F405
        return self.niScope_reset_cfunc(vi)

    def niScope_self_test(self, vi, self_test_result, self_test_message):  # noqa: N802
        with self._func_lock:
            if self.niScope_self_test_cfunc is None:
                self.niScope_self_test_cfunc = self._library.niScope_self_test
                self.niScope_self_test_cfunc.argtypes = [ViSession, ctypes.POINTER(ViInt16), ctypes.POINTER(ViChar)]  # noqa: F405
                self.niScope_self_test_cfunc.restype = ViStatus  # noqa: F405
        return self.niScope_self_test_cfunc(vi, self_test_result, self_test_message)



# -*- coding: utf-8 -*-
# This file was generated
import niscope._visatype as _visatype
import niscope.errors as errors

import array
import collections
import hightime
import numbers

from functools import singledispatch


@singledispatch
def _convert_repeated_capabilities(arg, prefix):  # noqa: F811
    '''Base version that should not be called

    Overall purpose is to convert the repeated capabilities to a list of strings with prefix from what ever form

    Supported types:
    - str - List (comma delimited)
    - str - Range (using '-' or ':')
    - str - single item
    - int
    - tuple
    - range
    - slice

    Each instance should return a list of strings, without prefix
    - '0' --> ['0']
    - 0 --> ['0']
    - '0, 1' --> ['0', '1']
    - 'ScriptTrigger0, ScriptTrigger1' --> ['0', '1']
    - '0-1' --> ['0', '1']
    - '0:1' --> ['0', '1']
    - '0-1,4' --> ['0', '1', '4']
    - range(0, 2) --> ['0', '1']
    - slice(0, 2) --> ['0', '1']
    - (0, 1, 4) --> ['0', '1', '4']
    - ('0-1', 4) --> ['0', '1', '4']
    - (slice(0, 1), '2', [4, '5-6'], '7-9', '11:14', '16, 17') -->
        ['0', '2', '4', '5', '6', '7', '8', '9', '11', '12', '13', '14', '16', '17']
    '''
    raise errors.InvalidRepeatedCapabilityError('Invalid type', type(arg))


@_convert_repeated_capabilities.register(numbers.Integral)  # noqa: F811
def _(repeated_capability, prefix):
    '''Integer version'''
    return [str(repeated_capability)]


# This parsing function duplicate the parsing in the driver, so if changes to the allowed format are made there, they will need to be replicated here.
@_convert_repeated_capabilities.register(str)  # noqa: F811
def _(repeated_capability, prefix):
    '''String version (this is the most complex)

    We need to deal with a range ('0-3' or '0:3'), a list ('0,1,2,3') and a single item
    '''
    # First we deal with a list
    rep_cap_list = repeated_capability.split(',')
    if len(rep_cap_list) > 1:
        # We have a list so call ourselves again to let the iterable instance handle it
        return _convert_repeated_capabilities(rep_cap_list, prefix)

    # Now we deal with ranges
    # We remove any prefix and change ':' to '-'
    r = repeated_capability.strip().replace(prefix, '').replace(':', '-')
    rc = r.split('-')
    if len(rc) > 1:
        if len(rc) > 2:
            raise errors.InvalidRepeatedCapabilityError("Multiple '-' or ':'", repeated_capability)
        start = int(rc[0])
        end = int(rc[1])
        if end < start:
            rng = range(start, end - 1, -1)
        else:
            rng = range(start, end + 1)
        return _convert_repeated_capabilities(rng, prefix)

    # If we made it here, it must be a simple item so we remove any prefix and return
    return [repeated_capability.replace(prefix, '').strip()]


# We cannot use collections.abc.Iterable here because strings are also iterable and then this
# instance is what gets called instead of the string one.
@_convert_repeated_capabilities.register(list)  # noqa: F811
@_convert_repeated_capabilities.register(range)  # noqa: F811
@_convert_repeated_capabilities.register(tuple)  # noqa: F811
def _(repeated_capability, prefix):
    '''Iterable version - can handle lists, ranges, and tuples'''
    rep_cap_list = []
    for r in repeated_capability:
        rep_cap_list += _convert_repeated_capabilities(r, prefix)
    return rep_cap_list


@_convert_repeated_capabilities.register(slice)  # noqa: F811
def _(repeated_capability, prefix):
    '''slice version'''
    def ifnone(a, b):
        return b if a is None else a
    # Turn the slice into a list and call ourselves again to let the iterable instance handle it
    rng = range(ifnone(repeated_capability.start, 0), repeated_capability.stop, ifnone(repeated_capability.step, 1))
    return _convert_repeated_capabilities(rng, prefix)


def convert_repeated_capabilities(repeated_capability, prefix=''):
    '''Convert a repeated capabilities object to a comma delimited list

    Args:
        repeated_capability (str, list, tuple, slice, None) -
        prefix (str) - common prefix for all strings

    Returns:
        rep_cap_list (list of str) - list of each repeated capability item with ranges expanded and prefix added
    '''
    # We need to explicitly handle None here. Everything else we can pass on to the singledispatch functions
    if repeated_capability is None:
        return []
    return [prefix + r for r in _convert_repeated_capabilities(repeated_capability, prefix)]


def convert_repeated_capabilities_without_prefix(repeated_capability):
    '''Convert a repeated capabilities object, without any prefix, to a comma delimited list

    Args:
        repeated_capability - Supported types:
            - str - list (comma-delimited)
            - str - range (using '-' or ':')
            - str - single item
            - int
            - list of str
            - tuple of str
            - range of str
            - slice of str
            - None

    Returns:
        rep_cap (str) - comma delimited string of each repeated capability item with ranges expanded
    '''
    return ','.join(convert_repeated_capabilities(repeated_capability, ''))


def _convert_timedelta(value, library_type, scaling):
    try:
        # We first assume it is a timedelta object
        scaled_value = value.total_seconds() * scaling
    except AttributeError:
        # If that doesn't work, assume it is a value in seconds
        # cast to float so scaled_value is always a float. This allows `timeout=10` to work as expected
        scaled_value = float(value) * scaling

    # ctype integer types don't convert to int from float so we need to
    if library_type in [_visatype.ViInt64, _visatype.ViInt32, _visatype.ViUInt32, _visatype.ViInt16, _visatype.ViUInt16, _visatype.ViInt8]:
        scaled_value = int(scaled_value)

    return library_type(scaled_value)


def convert_timedelta_to_seconds_real64(value):
    return _convert_timedelta(value, _visatype.ViReal64, 1)


def convert_timedelta_to_milliseconds_int32(value):
    return _convert_timedelta(value, _visatype.ViInt32, 1000)


def convert_timedeltas_to_seconds_real64(values):
    return [convert_timedelta_to_seconds_real64(i) for i in values]


def convert_seconds_real64_to_timedelta(value):
    return hightime.timedelta(seconds=value)


def convert_seconds_real64_to_timedeltas(values):
    return [convert_seconds_real64_to_timedelta(i) for i in values]


def convert_month_to_timedelta(months):
    return hightime.timedelta(days=(30.4167 * months))


# This converter is not called from the normal codegen path for function. Instead it is
# call from init and is a special case.
def convert_init_with_options_dictionary(values):
    if type(values) is str:
        init_with_options_string = values
    else:
        good_keys = {
            'rangecheck': 'RangeCheck',
            'queryinstrstatus': 'QueryInstrStatus',
            'cache': 'Cache',
            'simulate': 'Simulate',
            'recordcoercions': 'RecordCoercions',
            'interchangecheck': 'InterchangeCheck',
            'driversetup': 'DriverSetup',
            'range_check': 'RangeCheck',
            'query_instr_status': 'QueryInstrStatus',
            'record_coercions': 'RecordCoercions',
            'interchange_check': 'InterchangeCheck',
            'driver_setup': 'DriverSetup',
        }
        init_with_options = []
        for k in sorted(values.keys()):
            value = None
            if k.lower() in good_keys and not good_keys[k.lower()] == 'DriverSetup':
                value = good_keys[k.lower()] + ('=1' if values[k] is True else '=0')
            elif k.lower() in good_keys and good_keys[k.lower()] == 'DriverSetup':
                if not isinstance(values[k], dict):
                    raise TypeError('DriverSetup must be a dictionary')
                value = 'DriverSetup=' + (';'.join([key + ':' + values[k][key] for key in sorted(values[k])]))
            else:
                value = k + ('=1' if values[k] is True else '=0')

            init_with_options.append(value)

        init_with_options_string = ','.join(init_with_options)

    return init_with_options_string


# convert value to bytes
@singledispatch
def _convert_to_bytes(value):  # noqa: F811
    pass


@_convert_to_bytes.register(list)  # noqa: F811
@_convert_to_bytes.register(bytes)  # noqa: F811
@_convert_to_bytes.register(bytearray)  # noqa: F811
@_convert_to_bytes.register(array.array)  # noqa: F811
def _(value):
    return value


@_convert_to_bytes.register(str)  # noqa: F811
def _(value):
    return value.encode()


def convert_to_bytes(value):  # noqa: F811
    return bytes(_convert_to_bytes(value))


def convert_comma_separated_string_to_list(comma_separated_string):
    return [x.strip() for x in comma_separated_string.split(',')]


def convert_chained_repeated_capability_to_parts(chained_repeated_capability):
    '''Convert a chained repeated capabilities string to a list of comma-delimited repeated capabilities string.

    Converter assumes that the input contains the full cartesian product of its parts.
    e.g. If chained_repeated_capability is 'site0/PinA,site0/PinB,site1/PinA,site1/PinB',
    ['site0,site1', 'PinA,PinB'] is returned.

    Args:
        chained_repeated_capability (str) - comma-delimited repeated capabilities string where each
        item is a chain of slash-delimited repeated capabilities

    Returns:
        rep_cap_list (list of str) - list of comma-delimited repeated capabilities string
    '''
    chained_repeated_capability_items = convert_comma_separated_string_to_list(chained_repeated_capability)
    repeated_capability_lists = [[] for _ in range(chained_repeated_capability_items[0].count('/') + 1)]
    for item in chained_repeated_capability_items:
        repeated_capability_lists = [x + [y] for x, y in zip(repeated_capability_lists, item.split('/'))]
    return [','.join(collections.OrderedDict.fromkeys(x)) for x in repeated_capability_lists]





class MeasurementStats(object):
    def __init__(self, result=0.0, mean=0.0, stdev=0.0, min_val=0.0, max_val=0.0, num_in_stats=0):
        self.result = result
        self.mean = mean
        self.stdev = stdev
        self.min_val = min_val
        self.max_val = max_val
        self.num_in_stats = num_in_stats

        self.channel = None
        self.record = None

    def __repr__(self):
        parameter_list = [
            'result={}'.format(self.result),
            'mean={}'.format(self.mean),
            'stdev={}'.format(self.stdev),
            'min_val={}'.format(self.min_val),
            'max_val={}'.format(self.max_val),
            'num_in_stats={}'.format(self.num_in_stats)
        ]

        return '{0}({1})'.format(self.__class__.__name__, ', '.join(parameter_list))

    def __str__(self):
        row_format_g = '{:<20}: {:,.6g}\n'
        row_format_d = '{:<20}: {:,}\n'
        row_format_s = '{:<20}: {:}\n'

        string_representation = ''
        if self.channel is not None:
            string_representation += row_format_s.format('Channel', self.channel)
        if self.record is not None:
            string_representation += row_format_d.format('Record', self.record)

        string_representation += row_format_g.format('Result', self.result)
        string_representation += row_format_g.format('Mean', self.mean)
        string_representation += row_format_g.format('Standard Deviation', self.stdev)
        string_representation += row_format_g.format('Minimum Value', self.min_val)
        string_representation += row_format_g.format('Maximum Value', self.max_val)
        string_representation += row_format_d.format('Number in Stats', self.num_in_stats)

        return string_representation
